from fastapi.responses import ORJSONResponse
from sredis.sredis import get_datacenter_token, rr_get
from fastapi import HTTPException
import requests
import json
from core.models import ResourcePoolSchema


def proxy_datacenters(
    serverpath: str,
    username,
    method="GET",
    body="",
    balancer_fun=rr_get,
    datacenter=None,
):
    if "datacenter" in body:
        server = body["datacenter"]
    elif datacenter:
        server = datacenter
    else:
        server = balancer_fun()
    token = get_datacenter_token(username, server)
    url = f"{server}/{serverpath}"
    try:
        t_resp = requests.request(
            json=body,
            method=method,
            url=url,
            allow_redirects=True,
            verify=False,
            headers={"Authorization": token},
        )
        if int(t_resp.status_code / 100) != 2:
            raise HTTPException(
                status_code=t_resp.status_code, detail="Remote server error"
            )
        if not t_resp.content:
            return ORJSONResponse({"status": "ok"}, status_code=t_resp.status_code)
        obj = json.loads(t_resp.content)
        if type(obj) is list:
            for o in obj:
                o["datacenter"] = server
        else:
            obj["datacenter"] = server
        response = ORJSONResponse(obj, status_code=t_resp.status_code)
        return response
    except Exception:
        raise HTTPException(status_code=500, detail="Remote server error")


def create_vm_from_template(
    datacenter: str, username: str, name: str, template_name: str
):
    token = get_datacenter_token(username, datacenter)
    body = {"template_name": template_name, "username": username, "name": name}
    try:
        t_resp = requests.request(
            json=body,
            method="POST",
            url=f"{datacenter}dashboard/acpi/bvm/",
            allow_redirects=True,
            verify=False,
            headers={"Authorization": token},
        )
        if int(t_resp.status_code / 100) != 2:
            raise HTTPException(
                status_code=t_resp.status_code, detail="Remote server error"
            )
        obj = json.loads(t_resp.content)
        obj["datacenter"] = datacenter
        return obj
    except Exception:
        raise HTTPException(status_code=500, detail="Remote server error")


def create_rp(username: str, rp: ResourcePoolSchema):
    pass
